/*
 * Copyright © 2024 Harsh Shandilya.
 * Use of this source code is governed by an MIT-style
 * license that can be found in the LICENSE file or at
 * https://opensource.org/licenses/MIT.
 */
package dev.msfjarvis.claw.common.bonsai.node

import androidx.compose.runtime.Composable
import dev.msfjarvis.claw.common.bonsai.BonsaiScope
import dev.msfjarvis.claw.common.bonsai.node.extension.ExpandableNode
import dev.msfjarvis.claw.common.bonsai.node.extension.ExpandableNodeHandler
import dev.msfjarvis.claw.common.bonsai.node.extension.SelectableNode
import dev.msfjarvis.claw.common.bonsai.node.extension.SelectableNodeHandler
import java.util.UUID

public typealias NodeComponent<T> = @Composable BonsaiScope<T>.(Node<T>) -> Unit

public sealed interface Node<T> {

  public val key: String

  public val content: T

  public val name: String

  public val depth: Int

  public val isSelected: Boolean

  public val iconComponent: NodeComponent<T>

  public val nameComponent: NodeComponent<T>
}

public class LeafNode<T>
internal constructor(
  override val content: T,
  override val depth: Int,
  override val key: String = UUID.randomUUID().toString(),
  override val name: String = content.toString(),
  override val iconComponent: NodeComponent<T> = { DefaultNodeIcon(it) },
  override val nameComponent: NodeComponent<T> = { DefaultNodeName(it) },
) : Node<T>, SelectableNode by SelectableNodeHandler()

public class BranchNode<T>
internal constructor(
  override val content: T,
  override val depth: Int,
  override val key: String = UUID.randomUUID().toString(),
  override val name: String = content.toString(),
  override val iconComponent: NodeComponent<T> = { DefaultNodeIcon(it) },
  override val nameComponent: NodeComponent<T> = { DefaultNodeName(it) },
) : Node<T>, SelectableNode by SelectableNodeHandler(), ExpandableNode by ExpandableNodeHandler()
