/*
 * Copyright © 2024 Harsh Shandilya.
 * Use of this source code is governed by an MIT-style
 * license that can be found in the LICENSE file or at
 * https://opensource.org/licenses/MIT.
 */
package dev.msfjarvis.claw.common.bonsai.tree

import androidx.compose.runtime.Composable
import androidx.compose.runtime.Composition
import androidx.compose.runtime.Immutable
import androidx.compose.runtime.Stable
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCompositionContext
import dev.msfjarvis.claw.common.bonsai.node.Node
import dev.msfjarvis.claw.common.bonsai.node.TreeApplier
import dev.msfjarvis.claw.common.bonsai.tree.extension.ExpandableTree
import dev.msfjarvis.claw.common.bonsai.tree.extension.ExpandableTreeHandler
import dev.msfjarvis.claw.common.bonsai.tree.extension.SelectableTree
import dev.msfjarvis.claw.common.bonsai.tree.extension.SelectableTreeHandler

@DslMarker private annotation class TreeMarker

@Immutable
@TreeMarker
public data class TreeScope
internal constructor(
  val depth: Int,
  internal val isExpanded: Boolean = false,
  internal val expandMaxDepth: Int = 0,
)

@Stable
public class Tree<T> internal constructor(public val nodes: List<Node<T>>) :
  ExpandableTree<T> by ExpandableTreeHandler(nodes),
  SelectableTree<T> by SelectableTreeHandler(nodes)

@Composable
public fun <T> tree(content: @Composable TreeScope.() -> Unit): Tree<T> {
  val applier = remember { TreeApplier<T>() }
  val compositionContext = rememberCompositionContext()
  val composition =
    remember(applier, compositionContext) { Composition(applier, compositionContext) }
  composition.setContent { TreeScope(depth = 0).content() }
  return remember(applier) { Tree(applier.children) }
}
