/*
 * Copyright © 2024 Harsh Shandilya.
 * Use of this source code is governed by an MIT-style
 * license that can be found in the LICENSE file or at
 * https://opensource.org/licenses/MIT.
 */
package dev.msfjarvis.claw.common.bonsai.tree.extension

import dev.msfjarvis.claw.common.bonsai.node.BranchNode
import dev.msfjarvis.claw.common.bonsai.node.LeafNode
import dev.msfjarvis.claw.common.bonsai.node.Node

public interface SelectableTree<T> {

  public val selectedNodes: List<Node<T>>

  public fun toggleSelection(node: Node<T>)

  public fun selectNode(node: Node<T>)

  public fun unselectNode(node: Node<T>)

  public fun clearSelection()
}

internal class SelectableTreeHandler<T>(private val nodes: List<Node<T>>) : SelectableTree<T> {

  override val selectedNodes: List<Node<T>>
    get() = nodes.filter { it.isSelected }

  override fun toggleSelection(node: Node<T>) {
    if (node.isSelected) unselectNode(node) else selectNode(node)
  }

  override fun selectNode(node: Node<T>) {
    node.setSelected(true)
  }

  override fun unselectNode(node: Node<T>) {
    node.setSelected(false)
  }

  override fun clearSelection() {
    selectedNodes.forEach { it.setSelected(false) }
  }

  private fun Node<T>.setSelected(isSelected: Boolean) {
    when (this) {
      is LeafNode -> setSelected(isSelected)
      is BranchNode -> setSelected(isSelected)
    }
  }
}
